##-----------------------------LICENSE NOTICE------------------------------------
##  This file is part of CPCtelera: An Amstrad CPC Game Engine 
##  Copyright (C) 2016 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU Lesser General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
##------------------------------------------------------------------------------

############################################################################
##                        CPCTELERA ENGINE                                ##
##                 Automatic image conversion file                        ##
##------------------------------------------------------------------------##
## This file is intended for users to automate tilemap conversion from    ##
## original files (like Tiled .tmx) into C-arrays.                        ##
##                                                                        ##
## Macro used for conversion is TMX2C, which has up to 4 parameters:      ##
##  (1): TMX file to be converted to C array                              ##
##  (2): C identifier for the generated C array                           ##
##  (3): Output folder for C and H files generated (Default same folder)  ##
##  (4): Aditional options (aditional modifiers for cpct_tmx2csv)         ##
##                                                                        ##
## Macro is used in this way (one line for each image to be converted):   ##
##  $(eval $(call TMX2C,(1),(2),(3),(4)))                                 ##
##                                                                        ##
## Important:                                                             ##
##  * Do NOT separate macro parameters with spaces, blanks or other chars.##
##    ANY character you put into a macro parameter will be passed to the  ##
##    macro. Therefore ...,src/sprites,... will represent "src/sprites"   ##
##    folder, whereas ...,  src/sprites,... means "  src/sprites" folder. ##
##  * You can omit parameters by leaving them empty.                      ##
##  * Parameter  (4) (Aditional options) is  optional and  generally not  ##
##    required.
############################################################################

## Conversion Example
##

## Convert img/tilemap.tmx to src/tilemap.c and src/tilemap.h
##		This file contains a tilemap created with Tiled that uses tiles
## in img/tiles.png. This macro will convert the tilemap into a C-array
## named g_tilemap, containing all the IDs of the tiles that are located 
## at each given location of the C-array.
##

#$(eval $(call TMX2C,img/tilemap.tmx,g_tilemap,src/))

#$(eval $(call TMX2C,assets/map01.tmx,g_map01,src/sprites/))
#$(eval $(call TMX2C,assets/map02.tmx,g_map02,src/sprites/))
#$(eval $(call TMX2C,assets/map03.tmx,g_map03,src/sprites/))
#$(eval $(call TMX2C,assets/map04.tmx,g_map04,src/sprites/))


#################
# TMX2C: General rule to convert TMX tilemaps into C arrays.
# Updates IMGCFILES and OBJS2CLEAN adding new C files that result from 
# tmx conversions
#
# $(1): TMX file to be converted to C array
# $(2): C identifier for the generated C array
# $(3): Output folder for C and H files generated (Default same folder)
# $(4): Aditional options (you can use this to pass aditional modifiers to cpct_tmx2csv)
#
define TMX2COMP6
$(eval T2C_C  := $(basename $(1)).c)
$(eval T2C_H  := $(basename $(1)).h)
$(eval T2C_NC := $(notdir $(T2C_C)))
$(eval T2C_NH := $(notdir $(T2C_H)))
$(eval T2C_OF := $(shell if [ ! "$(3)" = "" ]; then echo "-of $(3)"; else echo ""; fi))
$(eval T2C_C2 := $(shell if [ ! "$(3)" = "" ]; then A="$(3)"; A="$${A%%/}"; echo "$${A}/$(T2C_NC)"; else echo "$(T2C_C)"; fi))
$(eval T2C_H2 := $(shell if [ ! "$(3)" = "" ]; then A="$(3)"; A="$${A%%/}"; echo "$${A}/$(T2C_NH)"; else echo "$(T2C_H)"; fi))
$(eval T2C_CH := $(T2C_C2) $(T2C_H2))
.SECONDARY: $(T2C_CH)
$(T2C_CH): $(1)
	@$(call PRINT,$(PROJNAME),"Converting tilemap in $(1) into C-arrays...")
	./cpct_tmx2comp6.sh -gh -ci $(2) $(T2C_OF) $(4) $(1)
IMGCFILES  := $(T2C_C2) $(IMGCFILES)
OBJS2CLEAN := $(T2C_CH) $(OBJS2CLEAN)
endef


$(eval $(call TMX2COMP6,assets/map1u.tmx,g_map1u,src/maps))
$(eval $(call TMX2COMP6,assets/map1d.tmx,g_map1d,src/maps))
$(eval $(call TMX2COMP6,assets/map1r.tmx,g_map1r,src/maps))
$(eval $(call TMX2COMP6,assets/map1l.tmx,g_map1l,src/maps))

$(eval $(call TMX2COMP6,assets/map2ud.tmx,g_map2ud,src/maps))
$(eval $(call TMX2COMP6,assets/map2ud_2.tmx,g_map2ud2,src/maps))
$(eval $(call TMX2COMP6,assets/map2ud_3.tmx,g_map2ud3,src/maps))
$(eval $(call TMX2COMP6,assets/map2rl.tmx,g_map2rl,src/maps))
$(eval $(call TMX2COMP6,assets/map2ur.tmx,g_map2ur,src/maps))
$(eval $(call TMX2COMP6,assets/map2ul.tmx,g_map2ul,src/maps))
$(eval $(call TMX2COMP6,assets/map2ul_2.tmx,g_map2ul2,src/maps))
$(eval $(call TMX2COMP6,assets/map2dr.tmx,g_map2dr,src/maps))
$(eval $(call TMX2COMP6,assets/map2dl.tmx,g_map2dl,src/maps))


$(eval $(call TMX2COMP6,assets/map3url.tmx,g_map3url,src/maps))
$(eval $(call TMX2COMP6,assets/map3drl.tmx,g_map3drl,src/maps))
$(eval $(call TMX2COMP6,assets/map3udl.tmx,g_map3udl,src/maps))
$(eval $(call TMX2COMP6,assets/map3udl_2.tmx,g_map3udl2,src/maps))
$(eval $(call TMX2COMP6,assets/map3udr.tmx,g_map3udr,src/maps))

$(eval $(call TMX2COMP6,assets/map4.tmx,g_map4,src/maps))
$(eval $(call TMX2COMP6,assets/map4_2.tmx,g_map42,src/maps))




