#include "tninja.h"

void ninjaBorrar(TNinja* ninja, const void* ptilemap) {
    u8 *pvmem;
    u8 w = 3 + (ninja->x[2] & 1);
    u8 h = 6 + (ninja->y[2] & 3 ? 1 : 0);

    // Comprobamos que no se pase del mapa
    u8 tiley = (ninja->y[2]-MAP_Y) / 4;
    u8 tilex = (ninja->x[2] / 2);
    if (tiley+h >= MAP_H && tilex>=35)
        h--;

    // Borramos
    pvmem = cpct_getScreenPtr(screenBuf, 0, MAP_Y);
    cpct_etm_drawTileBox2x4(tilex, tiley, w, h, MAP_W, pvmem, ptilemap);
}

void ninjaDibujar(TNinja* ninja, const void* tablatrans) {
    u8 *pvmem;
    u8* x;
    u8* y;

    TFrame* f = ninja->frame;

    // Giramos
    if (f->mira != ninja->mira) {
        cpct_hflipSpriteM0(NINJA_W, NINJA_H, f->sprite);
        f->mira = ninja->mira;
    }

    // Pintamos
    pvmem = cpct_getScreenPtr(screenBuf, ninja->x[0], ninja->y[0]);
    cpct_drawSpriteMaskedAlignedTable(ninja->frame->sprite, pvmem, NINJA_W, NINJA_H, tablatrans);

    // Actualizamos el movimiento
    x = ninja->x +2;
    y = ninja->y +2;
    *x = *(x-1); --x; *x = *(x-1);
    *y = *(y-1); --y; *y = *(y-1);

    if(ninja->mover)
        --ninja->mover;
}


void ninjaMoverIzquierda(TNinja* ninja) {
    // No comprobamos si se sale de pantalla. Que el mapa tenga muros de finalización
    u8* tileIzquierda;
    tileIzquierda = obtenerTilePtr(ninja->x[0] - 1, ninja->y[0]+NINJA_H-1); // Comprobamos tile de más abajo

    /* if(ninja->x>0 && *tileIzquierda != 1){ */
    if(*tileIzquierda != colL){
        ninja->mira = MiraIzq;
        --ninja->x[0];
    } else {
        // Si hay mas de un tile, escalamos
        tileIzquierda = obtenerTilePtr(ninja->x[0] -1, ninja->y[0]+NINJA_H/2);
        if(*tileIzquierda == colL)
            ninjaAgarrar(ninja);
    }
    ninja->mover = SI;
}

void ninjaMoverDerecha(TNinja* ninja) {
    // No comprobamos si se sale de pantalla. Que el mapa tenga muros de finalización
    u8* tileDerecha;
    tileDerecha = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+NINJA_H-1);  // Comprobamos tile de más abajo

    /* if(ninja->x<MAX_X && *tileDerecha != 1) { */
    if(*tileDerecha != colR) {
        ninja->mira = MiraDer;
        ++ninja->x[0];
    } else {
        // Si hay mas de un tile, escalamos
        tileDerecha = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+NINJA_H/2);
        if(*tileDerecha == colR)
            ninjaAgarrar(ninja);
    }
    ninja->mover = SI;
}

void ninjaAgarrar(TNinja* ninja) {
    if (ninja->estado != EstaEscalando) {
        ninja->estado = EstaAgarrando;
        ninja->ciclo = 0;
    }
}

void ninjaMoverArriba(TNinja* ninja) {
    u8* tile;
    u8 col;
    u8* tileArriba = obtenerTilePtr(ninja->x[0] + NINJA_W/2, ninja->y[0] -1);

    if (ninja->mira == MiraDer) {
        tile = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+1);  // Comprobamos tile central
        col = colR;
    } else {
        tile = obtenerTilePtr(ninja->x[0] - 1, ninja->y[0]+1);  // Comprobamos tile central
        col = colL;
    }
    if(*tile == col) {
        if(*tileArriba != colU){
            --ninja->y[0];
            ninja->mover = SI;
        }
    }
}

void ninjaMoverAbajo(TNinja* ninja) {
    u8* tile;
    u8 col;
    u8* tileAbajo = obtenerTilePtr(ninja->x[0] + NINJA_W/2, ninja->y[0] + NINJA_H);

    if (ninja->mira == MiraDer) {
        tile = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+1);  // Comprobamos tile central
        col = colR;
    } else {
        tile = obtenerTilePtr(ninja->x[0] - 1, ninja->y[0]+1);  // Comprobamos tile central
        col = colL;
    }

    if(*tile == col) {
        if(*tileAbajo != colD){
            ++ninja->y[0];
            ninja->mover = SI;
        }
    }
}


void ninjaAnimar(TNinja* ninja, u8 estado, TFrame* frame) {
    ninja->estado = estado;
    ninja->ciclo = 0;
    ninja->frame = frame;
    ninja->mover = SI;
}


void ninjaMueveLateral(TNinja* ninja) {
    if (teclado & K_LEFT)
        ninjaMoverIzquierda(ninja);
    else if (teclado & K_RIGHT)
        ninjaMoverDerecha(ninja);
}

void ninjaAnimarFrame(TNinja* ninja, u8 numFrames, TFrame* frames, u8 estadoFinal) {
    u8 i = (++ninja->ciclo/PAUSA_ANIM);
    if (i<numFrames)
        ninja->frame = frames+i;
    else if (i==numFrames){
    /* else { */
        // Ver pq si quito este if ocupa menos, solo el if no el contenido
        ninja->frame = frames+(numFrames-1);
        ninja->estado = estadoFinal;
        ninja->ciclo = 0;
        ninja->mover = SI;
    }
}

void ninjaActualizarFrame(TNinja* ninja) {
    switch (ninja->estado) {
        case EstaParado:  { ninja->frame = ninja->fAnimacion+FRAME_FIN_ATERRIZAR; break; }
        case EstaAndando: { ninja->frame = ninja->fAnimacion+(FRAME_INI_ANDAR + ((++ninja->ciclo/PAUSA_ANIM) % NUM_FRAMES_ANDAR)); break; }
        case EstaEscalando: { ninja->frame = ninja->fAnimacion+(FRAME_INI_ESCALAR + ((++ninja->ciclo/PAUSA_ANIM) % NUM_FRAMES_ESCALAR)); break; }
        case EstaSaltando:
            {
                // Actualizo ciclo con y en actualizarEstado
                u8 i = (ninja->ciclo/PAUSA_ANIM);
                if (i<NUM_FRAMES_SALTAR)
                    ninja->frame = ninja->fAnimacion+(FRAME_INI_SALTAR+i);
                else
                    ninja->frame = ninja->fAnimacion+FRAME_FIN_SALTAR;
                ninja->mover = SI;
                break;
            }
        /* case EstaCayendo: { break; } */
        case EstaAterrizando:
            {
                ninjaAnimarFrame(ninja, NUM_FRAMES_ATERRIZAR, ninja->fAnimacion+FRAME_INI_ATERRIZAR, EstaAndando);
                break;
            }
        case EstaAgarrando:
            {
                ninjaAnimarFrame(ninja, NUM_FRAMES_AGARRAR, ninja->fAnimacion+FRAME_INI_AGARRAR, EstaEscalando);
                break;
            }
        case EstaPreAtacando:
            {
                ninjaAnimarFrame(ninja, NUM_FRAMES_PREATACAR, ninja->fAnimacion+FRAME_INI_PREATACAR, EstaAtacando);
                break;
            }
        case EstaAtacando:
            {
                ninjaAnimarFrame(ninja, NUM_FRAMES_ATACAR, ninja->fAnimacion+FRAME_INI_ATACAR, EstaPosAtacando);
                break;
            }
        case EstaPosAtacando:
            {
                ninjaAnimarFrame(ninja, NUM_FRAMES_PREATACAR, ninja->fAnimacion+FRAME_INI_PREATACAR, EstaAndando);
                break;
            }
        //case EstaPreAtacandoAgachado:
        //    {
        //        ninjaAnimarFrame(ninja, NUM_FRAMES_PREATACAR_AGACHADO, ninja->fAnimacion+FRAME_INI_PREATACAR_AGACHADO, EstaAtacandoAgachado);
        //        break;
        //    }
        //case EstaAtacandoAgachado:
        //    {
        //        ninjaAnimarFrame(ninja, NUM_FRAMES_ATACAR_AGACHADO, ninja->fAnimacion+FRAME_INI_ATACAR_AGACHADO, EstaPosAtacandoAgachado);
        //        break;
        //    }
        //case EstaPosAtacandoAgachado:
        //    {
        //        ninjaAnimarFrame(ninja, NUM_FRAMES_PREATACAR_AGACHADO, ninja->fAnimacion+FRAME_INI_PREATACAR_AGACHADO, EstaAgachado);
        //        break;
        //    }
        case EstaAgachado:
            {
                if(ninja->frame != ninja->fAnimacion+FRAME_AGACHADO) {
                    ninja->frame = ninja->fAnimacion+FRAME_AGACHADO;
                    ninja->mover = SI;
                }
                break;
            }
    }
}


void ninjaActualizarEstado(TNinja* ninja) {

    switch  (ninja->estado) {
        case EstaParado: 
        case EstaAndando: 
            {
                if(teclado & K_DOWN) {
                    //if(cpct_getBit(teclado, K_SPACE)) {  // Atacando agachado
                    //    ninja->estado = EstaPreAtacandoAgachado;
                    //    ninja->ciclo = 0;
                    //} else {  // Solo agachado
                        ninja->estado = EstaAgachado;
                    //}
                    ninja->mover = SI;
                } else {
                    ninjaMueveLateral(ninja);

                    // Aun estando avanzando puede saltar
                    if (teclado & K_LEFT || teclado & K_RIGHT) {
                        u8* tile;
                        u8* tile2;

                        if (ninja->estado == EstaParado) {
                            ninja->estado = EstaAndando;
                        }

                        // Buscamos tile de abajo, si no hay, caemos
                        tile = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+NINJA_H);
                        tile2 = obtenerTilePtr(ninja->x[0], ninja->y[0]+NINJA_H);
//                        tile = obtenerTilePtr(ninja->x[0] + NINJA_W-3, ninja->y[0]+NINJA_H);
//                        tile2 = obtenerTilePtr(ninja->x[0]+2, ninja->y[0]+NINJA_H);

                        if(*tile != colD && *tile2 != colD)
                            ninjaAnimar(ninja, EstaCayendo, ninja->fAnimacion+FRAME_CAYENDO);
                    } else {
                        if (ninja->estado == EstaAndando) {
                            ninja->estado = EstaParado;
                            ninja->mover = SI;
                        }
                    }

                    if (teclado & K_UP)
                        ninjaAnimar(ninja, EstaSaltando, ninja->fAnimacion+FRAME_INI_SALTAR);
                    else if (teclado & K_SPACE)
                        ninjaAnimar(ninja, EstaPreAtacando, ninja->fAnimacion+FRAME_INI_PREATACAR);
                }
                break;
            }

        case EstaEscalando:
            {
                u8* tileDer;
                u8* tileIzq;

                if (teclado & K_LEFT || teclado & K_RIGHT) {
                    ninjaMueveLateral(ninja);
                    // Si despues de movernos lateralmente, no tenemos tiles ni a izq ni a der, caemos
                    tileDer = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]);
                    tileIzq = obtenerTilePtr(ninja->x[0] - 1, ninja->y[0]);
                    if(*tileDer != colR && *tileIzq != colL)
                        ninjaAnimar(ninja, EstaCayendo, ninja->fAnimacion+FRAME_CAYENDO);
                }

                if (teclado & K_UP)
                    ninjaMoverArriba(ninja);
                else if (teclado & K_DOWN)
                    ninjaMoverAbajo(ninja);
                break;
            }

        case EstaSaltando:
            {
                u8 salto;
                u8* tile;

                salto = CICLOS_SALTAR - ninja->ciclo;

                ninjaMueveLateral(ninja);
                ninja->ciclo++;
                // Comprobamos colision arriba
                tile = obtenerTilePtr(ninja->x[0], ninja->y[0] - salto);
                if (*tile != colU) {
                    ninja->y[0] -= salto;
                    ninja->mover = SI;
                } else {
                    // Si hemos colisionado, terminamos de saltar
                    ninja->y[0] = ((ninja->y[0] - salto)/4)*4;
                    ninja->ciclo = CICLOS_SALTAR+1;
                    ninja->mover = SI;
                }

                if (ninja->ciclo > CICLOS_SALTAR) {
                    // Si hemos terminado de saltar
                    ninjaAnimar(ninja, EstaCayendo, ninja->fAnimacion+FRAME_CAYENDO);
                }

                break;
            }

        case EstaCayendo:
            {
                // Comprobar colision y, y poner en EstaAterrizando
                u8* tile;
                u8* tile2;

                ninjaMueveLateral(ninja);

                tile = obtenerTilePtr(ninja->x[0] + NINJA_W, ninja->y[0]+NINJA_H);  // Tile abajo derecha
                tile2 = obtenerTilePtr(ninja->x[0], ninja->y[0]+NINJA_H);  // Tile abajo izquierda

                if(*tile == colD || *tile2 == colD) {
                    ninja->y[0] = ((ninja->y[0] + NINJA_H)/4 * 4)-NINJA_H; // Ajusto al suelo
                    ninja->estado = EstaAterrizando;
                    ninja->ciclo = 0;
                } else {
                    // Muevo eje y
                    if (ninja->ciclo>3)
                        ninja->y[0] += 3;
                    else
                        ninja->y[0] += ninja->ciclo;
                    ninja->ciclo++;
                    ninja->mover = SI;
                }
                break;
            }

        case EstaAgachado:
            {
                // Si se agacha, solo puede venir de estar andando
                if(!(teclado & K_DOWN)) {
                    // Nos levantamos
                    ninja->estado = EstaParado;
                    ninja->mover = SI;
                } //else if(cpct_getBit(teclado, K_SPACE)) {  // Atacando agachado
                  //  ninjaAnimar(ninja, EstaAtacandoAgachado, ninja->fAnimacion+FRAME_INI_ATACAR_AGACHADO);
                //}
                break;
            }

        // Estas opciones actualizan a EstaAndando en ninjaActualizarFrame, ya que el estado dura unos frames determinados
        case EstaAterrizando:          { ninja->mover = SI; break; }
        case EstaAgarrando:            { ninja->mover = SI; break; }
        case EstaPreAtacando:          { ninja->mover = SI; break; }
        case EstaAtacando:             { ninja->mover = SI; break; }
        case EstaPosAtacando:          { ninja->mover = SI; break; }
        //case EstaPreAtacandoAgachado:  { ninja->mover = SI; break; }
        //case EstaAtacandoAgachado:     { ninja->mover = SI; break; }
        //case EstaPosAtacandoAgachado:  { ninja->mover = SI; break; }
    }
}
