#ifndef _TNINJA_H_
#define _TNINJA_H_

#include <cpctelera.h>
#include "world.h"

#define PAUSA_ANIM       4
#define CICLOS_SALTAR    7

extern enum {
    EstaParado = 0,
    EstaAndando,
    EstaSaltando,
    EstaCayendo,
    EstaAterrizando,
    EstaAgarrando,        // Animación para empezar a escalar
    EstaEscalando,
    EstaPreAtacando,      // Preparando ataque
    EstaAtacando,
    EstaPosAtacando,      // Terminando ataque
    EstaAgachado,
    EstaPreAtacandoAgachado,    // Preparando ataque
    EstaAtacandoAgachado,
    EstaPosAtacandoAgachado     // Terminando ataque

} EEstado;


typedef struct {
    u8 x[3],y[3];
    TFrame* frame;
    u8 ciclo;
    u8 mover;
    u8 mira;
    u8 estado;
    TFrame* fAnimacion;
    //NUEVOS VALORES
    // u8 sala;
    u8 visto;
    u8 t; //tiempo
    u8 alterado; //ALTERADO >>> 0 = tranquilo | 1 = oido, no visto, patrulla en 1 nivel ejeX
    // u8 visible;
    u8 vidas;
} TNinja;

#define NUM_FRAMES_ANDAR               5
#define NUM_FRAMES_SALTAR              3
#define NUM_FRAMES_ATERRIZAR           3
#define NUM_FRAMES_AGARRAR             4
#define NUM_FRAMES_ESCALAR             2
#define NUM_FRAMES_PREATACAR           2
#define NUM_FRAMES_ATACAR              2
#define NUM_FRAMES_PREATACAR_AGACHADO  2
#define NUM_FRAMES_ATACAR_AGACHADO     2

// +1 frame cayendo, +1 frame agachado, -1 frame comun
#define NUM_FRAMES_ANIMACION           NUM_FRAMES_ANDAR + NUM_FRAMES_SALTAR + NUM_FRAMES_ATERRIZAR + NUM_FRAMES_AGARRAR + NUM_FRAMES_ESCALAR + NUM_FRAMES_PREATACAR+ NUM_FRAMES_ATACAR + NUM_FRAMES_PREATACAR_AGACHADO + NUM_FRAMES_ATACAR_AGACHADO  + 1 + 1 - 1

#define FRAME_INI_ANDAR                 0
#define FRAME_FIN_ANDAR                 FRAME_INI_ANDAR + NUM_FRAMES_ANDAR -1
#define FRAME_INI_SALTAR                5
#define FRAME_FIN_SALTAR                FRAME_INI_SALTAR + NUM_FRAMES_SALTAR -1
#define FRAME_INI_ATERRIZAR             8
#define FRAME_FIN_ATERRIZAR             FRAME_INI_ATERRIZAR + NUM_FRAMES_ATERRIZAR -1
#define FRAME_INI_AGARRAR               11
#define FRAME_FIN_AGARRAR               FRAME_INI_AGARRAR + NUM_FRAMES_AGARRAR -1
#define FRAME_INI_ESCALAR               14
#define FRAME_FIN_ESCALAR               FRAME_INI_ESCALAR + NUM_FRAMES_ESCALAR -1
#define FRAME_INI_PREATACAR             16
#define FRAME_FIN_PREATACAR             FRAME_INI_PREATACAR + NUM_FRAMES_PREATACAR -1
#define FRAME_INI_ATACAR                18
#define FRAME_FIN_ATACAR                FRAME_INI_ATACAR + NUM_FRAMES_ATACAR -1
#define FRAME_CAYENDO                   20
#define FRAME_AGACHADO                  21
#define FRAME_INI_PREATACAR_AGACHADO    21
#define FRAME_FIN_PREATACAR_AGACHADO    FRAME_INI_PREATACAR_AGACHADO + NUM_FRAMES_PREATACAR_AGACHADO -1
#define FRAME_INI_ATACAR_AGACHADO       23
#define FRAME_FIN_ATACAR_AGACHADO       FRAME_INI_ATACAR_AGACHADO + NUM_FRAMES_ATACAR_AGACHADO -1


#define NINJA_W G_NINJA_00_W
#define NINJA_H G_NINJA_00_H



void ninjaBorrar(TNinja* ninja, const void* ptilemap);
void ninjaDibujar(TNinja* ninja, const void* tablatrans);

void ninjaMoverIzquierda(TNinja* ninja);
void ninjaMoverDerecha(TNinja* ninja);

void ninjaAgarrar(TNinja* ninja);

void ninjaActualizarFrame(TNinja* ninja);
void ninjaActualizarEstado(TNinja* ninja);

void ninjaAnimar(TNinja* ninja, u8 estado, TFrame* frame);
void ninjaMueveLateral(TNinja* ninja);

#endif
